import fs from 'fs';
import { processConfig } from '@commercetools-frontend/application-config';
import _defineProperty from '@babel/runtime-corejs3/helpers/esm/defineProperty';
import _slicedToArray from '@babel/runtime-corejs3/helpers/esm/slicedToArray';
import _Array$isArray from '@babel/runtime-corejs3/core-js-stable/array/is-array';
import _reduceInstanceProperty from '@babel/runtime-corejs3/core-js-stable/instance/reduce';
import _Object$assign from '@babel/runtime-corejs3/core-js-stable/object/assign';
import _Object$keys from '@babel/runtime-corejs3/core-js-stable/object/keys';
import _mapInstanceProperty from '@babel/runtime-corejs3/core-js-stable/instance/map';
import _Object$entries from '@babel/runtime-corejs3/core-js-stable/object/entries';
import _trimInstanceProperty from '@babel/runtime-corejs3/core-js-stable/instance/trim';
import _concatInstanceProperty from '@babel/runtime-corejs3/core-js-stable/instance/concat';
import _Object$getOwnPropertySymbols from '@babel/runtime-corejs3/core-js-stable/object/get-own-property-symbols';
import _filterInstanceProperty from '@babel/runtime-corejs3/core-js-stable/instance/filter';
import _Object$getOwnPropertyDescriptor from '@babel/runtime-corejs3/core-js-stable/object/get-own-property-descriptor';
import _forEachInstanceProperty from '@babel/runtime-corejs3/core-js-stable/instance/for-each';
import _Object$getOwnPropertyDescriptors from '@babel/runtime-corejs3/core-js-stable/object/get-own-property-descriptors';
import _Object$defineProperties from '@babel/runtime-corejs3/core-js-stable/object/define-properties';
import _Object$defineProperty from '@babel/runtime-corejs3/core-js-stable/object/define-property';
import { HTTP_SECURITY_HEADER_KEYS, HTTP_SECURITY_HEADERS } from '@commercetools-frontend/constants';
import crypto from 'crypto';
import serialize from 'serialize-javascript';
export { g as generateTemplate } from './generate-template-31dee161.esm.js';

function createAssetHash(content) {
  const sha256Hash = crypto.createHash('sha256').update(content).digest('base64');

  /**
   * NOTE:
   *   We prefix the hash function type as the browser
   *   needs it when validating the contents of a script against
   *   CSP headers sent.
   *   For more information head to: developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Security-Policy/script-src#Sources
   */
  return `sha256-${sha256Hash}`;
}

const sanitizeAppEnvironment = env => serialize(env, {
  isJSON: true
});

function ownKeys(e, r) { var t = _Object$keys(e); if (_Object$getOwnPropertySymbols) { var o = _Object$getOwnPropertySymbols(e); r && (o = _filterInstanceProperty(o).call(o, function (r) { return _Object$getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var _context7, _context8; var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? _forEachInstanceProperty(_context7 = ownKeys(Object(t), !0)).call(_context7, function (r) { _defineProperty(e, r, t[r]); }) : _Object$getOwnPropertyDescriptors ? _Object$defineProperties(e, _Object$getOwnPropertyDescriptors(t)) : _forEachInstanceProperty(_context8 = ownKeys(Object(t))).call(_context8, function (r) { _Object$defineProperty(e, r, _Object$getOwnPropertyDescriptor(t, r)); }); } return e; }
// https://babeljs.io/blog/2017/09/11/zero-config-with-babel-macros
const htmlScripts$1 = {
  "loadingScreen": "window.onAppLoaded=function(){var e=document.querySelector(\"#app-loader\");e&&e.parentNode.removeChild(e)},setTimeout(function(){var e=document.querySelector(\".loading-screen\");e&&e.classList.remove(\"loading-screen--hidden\")},250),setTimeout(function(){var e=document.querySelector(\".long-loading-notice\");e&&e.classList.remove(\"long-loading-notice--hidden\")},2e3);",
  "publicPath": "window.__dynamicImportHandler__=function(n){return window.app.cdnUrl.replace(/\\/$/,\"\")+\"/\"+n.replace(/^(\\.\\/)?/,\"\")},window.__dynamicImportPreload__=function(n){return n.map(n=>window.app.cdnUrl.replace(/\\/$/,\"\")+\"/\"+n)};"
};
const toArray = value => _Array$isArray(value) ? value : [value];
const parseCSPDirectives = function () {
  var _context2;
  for (var _len = arguments.length, directives = new Array(_len), _key = 0; _key < _len; _key++) {
    directives[_key] = arguments[_key];
  }
  const mergedDirectives = _reduceInstanceProperty(directives).call(directives, (mergedDirectives, directive) => {
    var _context;
    return _Object$assign(mergedDirectives, _reduceInstanceProperty(_context = _Object$keys(directive)).call(_context, (mergedDirectiveValues, directiveKey) => {
      const existingDirectiveValue = mergedDirectives[directiveKey];
      return _Object$assign(mergedDirectiveValues, {
        [directiveKey]: [...toArray(existingDirectiveValue ?? []), ...toArray(directive[directiveKey])]
      });
    }, {}));
  }, {});
  return _mapInstanceProperty(_context2 = _Object$entries(mergedDirectives)).call(_context2, _ref => {
    let _ref2 = _slicedToArray(_ref, 2),
      directive = _ref2[0],
      value = _ref2[1];
    return `${directive} ${_Array$isArray(value) ? value.join(' ') : value}`;
  }).join('; ');
};
const parsePermissionsPolicyDirectives = (defaultValue, customDirectives) => {
  var _context3;
  const mergedDirectives = _mapInstanceProperty(_context3 = defaultValue.split(',')).call(_context3, directive => {
    const _directive$trim$split = _trimInstanceProperty(directive).call(directive).split('='),
      _directive$trim$split2 = _slicedToArray(_directive$trim$split, 2),
      directiveName = _directive$trim$split2[0],
      directiveValue = _directive$trim$split2[1];
    const overrideDirectiveValue = customDirectives[directiveName];
    if (overrideDirectiveValue) {
      return [directiveName, overrideDirectiveValue].join('=');
    }
    return [directiveName, directiveValue].join('=');
  });
  return mergedDirectives.join(', ');
};
const processHeaders = applicationConfig => {
  var _context4, _context5, _context6;
  const isMcDevEnv = applicationConfig.env.env === 'development';

  // List hashes for injected inline scripts.
  // https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Security-Policy/script-src
  const htmlScriptsHashes = [createAssetHash(htmlScripts$1.loadingScreen), createAssetHash(`window.app = ${sanitizeAppEnvironment(applicationConfig.env)};`), createAssetHash(htmlScripts$1.publicPath)];

  // // List hashes for injected inline styles.
  // // https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Security-Policy/style-src
  // const htmlStylesHashes = [createAssetHash(htmlStyles.loadingScreen)];

  /**
   * Content Security Policy (CSP)
   * https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Security-Policy
   */
  const defaultCSPDirectives = _Object$assign({
    'default-src': "'none'",
    'script-src': _concatInstanceProperty(_context4 = ["'self'"]).call(_context4, isMcDevEnv ?
    // Allow webpack to load source maps on runtime when errors occur
    // using script tags
    ['localhost:*', "'unsafe-inline'"] : _mapInstanceProperty(htmlScriptsHashes).call(htmlScriptsHashes, assetHash => `'${assetHash}'`)),
    'connect-src': _concatInstanceProperty(_context5 = ["'self'", 'app.launchdarkly.com', 'clientstream.launchdarkly.com', 'events.launchdarkly.com', 'app.getsentry.com',
    // Match all attempts to load from any subdomain of `sentry.io`
    '*.sentry.io']).call(_context5, isMcDevEnv ? ['ws:', 'localhost:8080', 'webpack-internal:'] : []),
    'img-src': ['*', 'data:'],
    'style-src': _concatInstanceProperty(_context6 = ["'self'", 'fonts.googleapis.com', 'data:']).call(_context6,
    // TODO: investigate what needs to be done to avoid unsafe-inline styles
    // https://github.com/commercetools/merchant-center-frontend/pull/5223#discussion_r210367636
    ["'unsafe-inline'"]
    // TODO: enable this once we can avoid unsafe-inline
    // htmlStylesHashes.map(assetHash => `'${assetHash}'`)
    ),
    'font-src': ["'self'", 'fonts.gstatic.com', 'data:'],
    // Required for Custom Views
    'frame-src': ["'self'"]
  }, isMcDevEnv ? {
    // NOTE: use this instead of `upgrade-insecure-requests` for local
    // development to avoid `http://localhost` requests to be redirected
    // to https.
    'block-all-mixed-content': ''
  } : {
    // NOTE: prefer this over `block-all-mixed-content`.
    // https://youtu.be/j-0Bj40juMI?t=11m47s
    'upgrade-insecure-requests': ''
  }
  // NOTE: we might want to define further policies in the future, for example
  // - `require-sri-for style script` (at the moment not possible because
  //  Intercom scripts are apparently not meant for this)
  );
  return _objectSpread(_objectSpread({}, HTTP_SECURITY_HEADERS), {}, {
    // The `Content-Security-Policy` header is always generated
    // based on the Merchant Center customization config.
    [HTTP_SECURITY_HEADER_KEYS['Content-Security-Policy']]: parseCSPDirectives(defaultCSPDirectives, applicationConfig.headers?.csp ?? {})
  }, applicationConfig.headers?.permissionsPolicies ? {
    [HTTP_SECURITY_HEADER_KEYS['Permissions-Policy']]: parsePermissionsPolicyDirectives(HTTP_SECURITY_HEADERS['Permissions-Policy'], applicationConfig.headers.permissionsPolicies ?? {})
  } : {});
};

// https://babeljs.io/blog/2017/09/11/zero-config-with-babel-macros
const htmlScripts = {
  "loadingScreen": "window.onAppLoaded=function(){var e=document.querySelector(\"#app-loader\");e&&e.parentNode.removeChild(e)},setTimeout(function(){var e=document.querySelector(\".loading-screen\");e&&e.classList.remove(\"loading-screen--hidden\")},250),setTimeout(function(){var e=document.querySelector(\".long-loading-notice\");e&&e.classList.remove(\"long-loading-notice--hidden\")},2e3);",
  "publicPath": "window.__dynamicImportHandler__=function(n){return window.app.cdnUrl.replace(/\\/$/,\"\")+\"/\"+n.replace(/^(\\.\\/)?/,\"\")},window.__dynamicImportPreload__=function(n){return n.map(n=>window.app.cdnUrl.replace(/\\/$/,\"\")+\"/\"+n)};"
}; // https://babeljs.io/blog/2017/09/11/zero-config-with-babel-macros
const htmlStyles = {
  "loadingScreen": ".loading-screen{display:flex;flex-direction:column;align-items:center;justify-content:center;height:100vh;width:100vw}.loading-screen--hidden{display:none}.loading-screen>*+*{margin:24px 0 0}.loading-spinner{width:32px;height:32px}.long-loading-notice{color:#999;font-family:'Open Sans',sans-serif;font-size:12px}.long-loading-notice--hidden{visibility:hidden}.loading-spinner-circle{fill:#213c45;opacity:.2}@keyframes loading-spinner-animation{0%{transform:rotate(0)}100%{transform:rotate(360deg)}}.loading-spinner-pointer{transform-origin:20px 20px 0;animation:loading-spinner-animation .5s infinite linear}"
};
const trimTrailingSlash = value => value.replace(/\/$/, '');
const replaceHtmlPlaceholders = (indexHtmlContent, options) => indexHtmlContent.replace(new RegExp('__CSP__', 'g'), options.headers?.['Content-Security-Policy'] ?? '').replace(new RegExp('__CDN_URL__', 'g'), options.env.cdnUrl ?
// Ensure there is a trailing slash
`${trimTrailingSlash(options.env.cdnUrl)}/` : '').replace(new RegExp('__MC_API_URL__', 'g'), trimTrailingSlash(options.env.mcApiUrl)).replace(new RegExp('__APPLICATION_ENVIRONMENT__', 'g'), `<script>window.app = ${sanitizeAppEnvironment(options.env)};</script>`).replace(new RegExp('__LOADING_SCREEN_JS__', 'g'), `<script>${htmlScripts.loadingScreen}</script>`).replace(new RegExp('__LOADING_SCREEN_CSS__', 'g'), `<style>${htmlStyles.loadingScreen}</style>`);

async function compileHtml(indexHtmlTemplatePath) {
  const applicationConfig = await processConfig();
  const compiledHeaders = processHeaders(applicationConfig);
  const indexHtmlTemplateContent = fs.readFileSync(indexHtmlTemplatePath, 'utf8');
  const indexHtmlContent = replaceHtmlPlaceholders(indexHtmlTemplateContent, {
    env: applicationConfig.env,
    headers: compiledHeaders
  });
  return {
    env: applicationConfig.env,
    headers: compiledHeaders,
    indexHtmlContent
  };
}

export { compileHtml, processHeaders, replaceHtmlPlaceholders };
